unit SafeBitmap;

interface

uses SysUtils, Windows, Graphics, Classes, Compat;

{$IFDEF VER100}
{$ASSERTIONS ON}
{$ENDIF}

type
   TSafeBitmap = class(TComponent)
   protected
      FSafeBmp    : pointer;
      procedure   SetPixelFormat(pf: TPixelFormat);
      function    GetPixelFormat: TPixelFormat;
      procedure   Lock;
      procedure   Unlock;
   public
      constructor Create(AOwner: TComponent); override;
      destructor  Destroy; override;
      procedure   SetSize(x, y: integer);
      function    GetDC: HDC;
      function    LoadFromFile(fn: string): boolean;
      function    SaveToFile(fn: string): boolean;
      property    PixelFormat: TPixelFormat read GetPixelFormat write SetPixelFormat;
   end;

implementation

type
   TBitmapCreateFunc  = function: pointer; stdcall;
   TBitmapFreeFunc    = procedure (var bm: pointer); stdcall;
   TBitmapSetSizeFunc = procedure (bm: pointer; x, y: integer); stdcall;
   TBitmapGetDCFunc   = function  (bm: pointer): HDC; stdcall;
   TBitmapLoadFunc    = function  (bm: pointer; fn: PChar): boolean; stdcall;
   TBitmapSetPixelFormatFunc = procedure (bm: pointer; pf: TPixelFormat); stdcall;
   TBitmapGetPixelFormatFunc = function (bm: TBitmap): TPixelFormat; stdcall;


var
   SafeBmpCrit          : TRTLCriticalSection;
   hDLL                 : THandle;
   BitmapCreate         : TBitmapCreateFunc;
   BitmapFree           : TBitmapFreeFunc;
   BitmapSetSize        : TBitmapSetSizeFunc;
   BitmapGetDC          : TBitmapGetDCFunc;
   BitmapLoad           : TBitmapLoadFunc;
   BitmapSave           : TBItmapLoadFunc;
   BitmapSetPixelFormat : TBitmapSetPixelFormatFunc;
   BitmapGetPixelFormat : TBitmapGetPixelFormatFunc;

function InitDLL: boolean;
const
   DLLName = 'WABDSafeBitmap.dll';
begin
   hDLL := LoadLibrary(DLLName);

   if hDLL=0 then begin
      raise Exception.Create('LoadLibrary failed for: ' + DLLName);
   end;

   BitmapCreate := GetProcAddress(hDLL, 'BitmapCreate');
   BitmapFree := GetProcAddress(hDLL, 'BitmapFree');
   BitmapSetSize := GetProcAddress(hDLL, 'BitmapSetSize');
   BitmapGetDC := GetProcAddress(hDLL, 'BitmapGetDC');
   BitmapLoad := GetProcAddress(hDLL, 'BitmapLoad');
   BitmapSave := GetProcAddress(hDLL, 'BitmapSave');
   BitmapSetPixelFormat := GetProcAddress(hDLL, 'BitmapSetPixelFormat');
   BitmapGetPixelFormat := GetProcAddress(hDLL, 'BitmapGetPixelFormat');
   Result := True;
end;


constructor TSafeBitmap.Create(AOwner: TComponent);
begin
   Lock;

   if hDLL = 0 then InitDLL;
   inherited;
   FSafeBmp := BitmapCreate;

   Unlock;
end;

destructor TSafeBitmap.Destroy;
begin
   Lock;

   BitmapFree(FSafeBmp);
   inherited;

   Unlock;
end;

procedure TSafeBitmap.SetSize(x, y: integer);
begin
   Lock;
   BitmapSetSize(FSafeBmp, x, y);
   Unlock;
end;

procedure TSafeBitmap.SetPixelFormat(pf: TPixelFormat);
begin
   Lock;
   Assert(pf <> pf16bit, 'IE has trouble with 16-bit bitmaps!');
   Assert(pf <> pf15bit, 'IE has trouble with 15-bit bitmaps!');
   BitmapSetPixelFormat(FSafeBmp, pf);
   Unlock;
end;

function TSafeBitmap.GetPixelFormat: TPixelFormat;
begin
   Lock;
   Result := BitmapGetPixelFormat(FSafeBmp);
   Unlock;
end;

function TSafeBitmap.GetDC: HDC;
begin
   Lock;
   Result := BitmapGetDC(FSafeBmp);
   Unlock;
end;

function TSafeBitmap.LoadFromFile(fn: string): boolean;
begin
   Lock;
   Result := BitmapLoad(FSafeBmp, PChar(fn));
   Unlock;
end;

function TSafeBitmap.SaveToFile(fn: string): boolean;
begin
   Lock;
   Result := BitmapSave(FSafeBmp, PChar(fn));
   Unlock;
end;

procedure TSafeBitmap.Lock;
begin
   EnterCriticalSection(SafeBmpCrit);
end;

procedure TSafeBitmap.Unlock;
begin
   LeaveCriticalSection(SafeBmpCrit);
end;


initialization
   hDLL := 0;
   InitializeCriticalSection(SafeBmpCrit);
finalization
   DeleteCriticalSection(SafeBmpCrit);
end.

